#!/usr/bin/env python
# coding=utf-8
"""
for example:
    from file_utils import YamlFileUtils, JsonFileUtils, CsvFileUtils, ExcelFileUtils, IniFileUtils, CSVFileUtils
    yaml_file = YamlFileUtils()
    data = yaml_file.read('data.yaml')
    print(data)
    yaml_file.write('data.yaml', {'name': 'zhangsan', 'age': 18})
    >>>
    {'name': 'zhangsan', 'age': 18}

    json_file = JsonFileUtils()
    data = json_file.read('data.json')
    print(data)
    json_file.write('data.json', {'name': 'zhangsan', 'age': 18})
    >>>
    {'name': 'zhangsan', 'age': 18}

    excel_file_utils = ExcelFileUtils()
    excel_file_utils.write('test.xlsx', [['a', 'b', 'c'], [1, 2, 3]])
    print(excel_file_utils.read('test.xlsx'))
    >>>
    [['a', 'b', 'c'], [1.0, 2.0, 3.0]]

    csv_file_utils = CSVFileUtils()
    csv_file_utils.write('test.csv', [['a', 'b', 'c'], [1, 2, 3]])
    print(csv_file_utils.read('test.csv'))
    >>>
    [['a', 'b', 'c'], ['1', '2', '3']]

    inifile = IniFileUtils()
    inifile.write('config.ini', {'section1': {'key1': 'value1', 'key2': 'value2'}})
    data = inifile.read('config.ini')
    print(data)
    >>>
    {'section1': {'key1': 'value1', 'key2': 'value2'}}
"""


import json, yaml, xlrd, csv, configparser
from abc import ABCMeta, abstractmethod
from typing import Dict, List, Tuple, Union, Any
from openpyxl import workbook


class FileUtils(metaclass=ABCMeta):
    @abstractmethod
    def read(self, file_path: str):
        """
        抽象方法，用于读取文件
        :param file_path: 文件路径
        :return: 文件内容，可能是字符串或字节数组
        """
        pass

    @abstractmethod
    def write(self, file_path: str, content: Union[List, Dict, Tuple, str, bytes], mode: str = 'w'):
        """
        抽象方法，用于写入文件
        :param file_path: 文件路径
        :param content: 要写入的内容，可以是列表、字典、元组、字符串或字节数组
        :param mode: 写入模式，默认为'w'，表示覆盖写入
        """
        pass

class JsonFileUtils(FileUtils):
    def read(self, file_path) -> Dict[Any, Any]:
        with open(file_path, 'r', encoding='utf-8') as f:
            return json.load(f)

    def write(self, file_path, content: Dict, mode='w'):
        """
        将内容写入json文件

        Args:
            file_path (str): 文件路径
            content (Dict): 要写入的内容
            mode (str, optional): 写入模式，默认为'w'，表示覆盖写入. Defaults to 'w'.
        """
        with open(file_path, 'w', encoding='utf-8') as f:
            json.dump(content, f, indent=4, ensure_ascii=False)

class YamlFileUtils(FileUtils):
    def read(self, file_path) -> Union[List[Any], Dict[Any, Any]]:
        with open(file_path, 'r', encoding='utf-8') as f:
            return yaml.load(f, Loader=yaml.FullLoader)

    def write(self, file_path, content: Union[Dict, List, Tuple, str], mode='w'):
        """
        将数据写入到yaml文件中

        Args:
            file_path: 文件路径
            content: 要写入的数据，可以是字典、列表、元组等
            mode: 写入模式，默认为'w'
        """
        with open(file_path, mode=mode, encoding='utf-8') as f:
            yaml.dump(content, f, allow_unicode=True)  # 写入的中文不进行转码

class ExcelFileUtils(FileUtils):
    def read(self, file_path) -> List[List]:
        with xlrd.open_workbook(file_path) as f:
            sheet = f.sheet_by_index(0)
            rows = sheet.nrows
            cols = sheet.ncols
            data = []
            for row in range(rows):
                row_data = []
                for col in range(cols):
                    row_data.append(sheet.cell_value(row, col))
                data.append(row_data)
            return data
           
    def write(self, file_path, content: list[list]):
        """
        将数据写入 Excel 文件

        Args:
            file_path (str): 文件保存路径     
            content (list[list]): 要写入的数据，二维列表
        """
        wb = workbook.Workbook()
        sheet = wb.active

        for row, row_data in enumerate(content):
            for col, value in enumerate(row_data):
                sheet.cell(row=row + 1, column=col + 1, value=value)

        wb.save(file_path)

class CSVFileUtils(FileUtils):
    def read(self, file_path):
        with open(file_path, 'r', encoding='utf-8') as f:
            reader = csv.reader(f)
            data = [row for row in reader]
            return data

    def write(self, file_path, content: list[list], mode='w'):
        """
        将数据写入 CSV 文件

        Args:
            file_path (str): 文件保存路径
            content (list[list]): 要写入的数据，二维列表
            mode (str): 文件打开模式，默认为 'w'
        """
        with open(file_path, mode=mode, encoding='utf-8', newline='') as f:
            writer = csv.writer(f)
            writer.writerows(content)
            
class IniFileUtils(FileUtils):
    config = configparser.ConfigParser()

    def read(self, file_path):
        self.config.read(file_path, encoding='utf-8')
        data = {}
        for section in self.config.sections():
            data[section] = {}
            for key, value in self.config.items(section):
                data[section][key] = value
        return data 
    
    def write(self, file_path, content: Dict[Any, Dict[Any, Any]], mode='w'):
        for section, values in content.items():
            self.config[section] = values
        with open(file_path, mode=mode, encoding='utf-8') as f:
            self.config.write(f)




#!/usr/bin/env python
# coding=utf-8
""""
for example:
    from decorator_utils import DecoratorUtils, LogUtils

    logger = LogUtils().get_logger()
    decorator = DecoratorUtils(logger)
    @decorator.logger_decorator
    def func():
        pass
    >>>
    2024-08-02 10:39:13,382 - app - INFO - Kwargs: {'url': 'www.baidu.com', 'name': 'test'
    2024-08-02 10:39:13,382 - app - INFO - Args: ('afghhf', 'fghfghfgh')
"""
import time
from functools import wraps
from log_utils import LogUtils


class DecoratorUtils:
    """
    日志工具类，用于记录函数的输入输出和执行时间
    """
    def __init__(self, logger: LogUtils):
        self.logger = logger

    def logger_decorator(self, func):
        """
        日志装饰器，用于记录函数的调用信息

        参数:
        - func: 要装饰的函数

        返回:
        - 装饰后的函数
        """
        try:
            @wraps(func)
            def wrapper(*args, **kwargs):
                if args:
                    self.logger.info(f"Args: {args}")
                if kwargs:
                    self.logger.info(f"Kwargs: {kwargs}")
                return func(*args, **kwargs)

            return wrapper
        except Exception as e:
            self.logger.error(f"Error occurred in {func.__name__}: {str(e)}")
            raise e
        
    def time_decorator(self, func):
        """
        计时装饰器，用于记录函数的执行时间

        参数:
        - func: 要装饰的函数

        返回:
        - 装饰后的函数
        """
        try:
            @wraps(func)
            def wrapper(*args, **kwargs):
                start_time = time.time()
                result = func(*args, **kwargs)
                end_time = time.time()
                print(f"cost time: {end_time - start_time} seconds")
                return result
            
            return wrapper
        except Exception as e:
            raise e



#!/usr/bin/env python
# coding=utf-8
import os
import pymysql
from dotenv import load_dotenv, find_dotenv
from abc import ABCMeta, abstractmethod
from typing import Union, List, Dict, Tuple, Any
from sshtunnel import SSHTunnelForwarder

"""
for example:
    env_db = MysqlUtils(database='env')    # 使用环境变量

    base = MysqlUtils(database='base', host='localhost', user='root', password='123456')
    print(base)
    print(get_db_data(base, 'select * from user'))

    >>>
    [{'id': 1, 'name': 'hejia', 'created_at': datetime.datetime(2024, 7, 29, 16, 12, 56, 11808),'updated_at': datetime.datetime(2024, 7, 29, 16, 12, 56, 11808), 'deleted_at': None, 'is_deleted': 0, 'is_superuser': 0, 'password': 'abc123456', 'phone': '1352625236'}, 
    {'id': 2, 'name': 'hejia24', 'created_at': datetime.datetime(2024, 7, 29, 16, 15, 24, 501265),'updated_at': datetime.datetime(2024, 7, 29, 16, 15, 24, 501265), 'deleted_at': None, 'is_deleted': 0, 'is_superuser': 0, 'password': 'abc123456', 'phone': None}]
"""


class DBUtils(metaclass=ABCMeta):
    def __init__(self, database:str, is_ssh: bool=False, curson_type: bool=True, user_dotenv: bool=True, **kwargs):
        """
        初始化数据库连接
        :param database: 数据库名称
        :param is_ssh: 是否使用SSH隧道
        :param curson_type: 是否使用字典游标
        :param user_dotenv: 是否使用环境变量
        :param kwargs: 其他参数 
        """
        if user_dotenv:
            # 加载环境变量
            load_dotenv(find_dotenv(), override=True)
            self.host = os.getenv("DB_HOST", "localhost")
            self.port = int(os.getenv("DB_PORT", 3306))
            self.user = os.getenv("DB_USER", "root")
            self.password = os.getenv("DB_PASSWORD", "123456")
            self.ssh_host = os.getenv("SSH_HOST", None)
            self.ssh_port = int(os.getenv("SSH_PORT", 22))
            self.ssh_user = os.getenv("SSH_USER", None)
            self.ssh_password = os.getenv("SSH_PASSWORD", None)
        else:
            if kwargs:
                db_attr = ["host", "port", "user", "password", "ssh_host", "ssh_port", "ssh_user", "ssh_password"]
                __setattr = [setattr(self, arg, kwargs[arg]) for arg in kwargs if arg in db_attr]

        self.database = database
        self.is_ssh = is_ssh
        self.cursor_type = curson_type
    
    @abstractmethod
    def connect(self):
        pass

    @abstractmethod
    def execute(self, sql) -> None:
        pass

    @abstractmethod
    def select_one(self) -> Union[None, Dict[str, Any], Tuple[Any]]:
        pass

    @abstractmethod
    def select_all(self) -> Union[None, List[Dict[str, Any]], Tuple[Tuple[Any]]]:
        pass

    @abstractmethod
    def close(self):
        pass
    
    def __str__(self) -> str:
        attrs = vars(self)
        return '\n'.join("%s: %s" % item for item in attrs.items())
    
class MysqlUtils(DBUtils):
    def __init__(self, database:str, is_ssh: bool=False, curson_type: bool=True, user_dotenv: bool=True, **kwargs):
        super().__init__(database, is_ssh, curson_type, user_dotenv, **kwargs)

    def connect(self) -> None:
        try:
            if self.is_ssh:
                with SSHTunnelForwarder(
                    # 指定ssh登录的跳转机的address
                    ssh_address_or_host=(self.ssh_host, int(self.ssh_port)),
                    ssh_username=self.ssh_user,
                    ssh_password=self.ssh_password,
                    # 设置数据库服务地址及端口
                    remote_bind_address=(self.host, int(self.port))) as server:
                    self.conn = pymysql.connect(
                        database=self.database,
                        user=self.user,
                        password=self.password,
                        host='127.0.0.1',
                        port=server.local_bind_port,
                        cursorclass=pymysql.cursors.DictCursor if self.cursor_type else pymysql.cursors.Cursor
            )
            else:
                self.conn = pymysql.connect(
                    host=self.host, port=self.port, user=self.user, password=self.password, database=self.database,
                    cursorclass=pymysql.cursors.DictCursor if self.cursor_type else pymysql.cursors.Cursor
            )
        except pymysql.MySQLError as e:
            print(f"数据库连接失败: {e}")

    def execute(self, sql) -> None:
        try:
            self.cursor = self.conn.cursor()
            self.cursor.execute(sql)
            self.conn.commit()
        except pymysql.MySQLError as e:
            print(f"查询失败: {e}")
        finally:
            print(f"执行SQL: {sql}")

    def select_one(self) -> Union[None, Dict[str, Any], Tuple[Any]]:
        return self.cursor.fetchone()

    def select_all(self) ->  Union[None, List[Dict[str, Any]], Tuple[Tuple[Any]]]:
        return self.cursor.fetchall()

    def close(self) -> None:
        self.cursor.close()
        self.conn.close()

def get_db_data(conn: DBUtils, sql: str, is_one: bool = False) -> Union[None, Dict[str, Any], Tuple[Any], Dict[str, Any], Tuple[Any]]:
    try:
        conn.connect()
        conn.execute(sql)
        if is_one:
            return conn.select_one()
        else:
            return conn.select_all()
    finally:
        conn.close()

if __name__ == '__main__':
    base = MysqlUtils(database='base', host='localhost', user='root', password='123456')
    print(base)
    print(get_db_data(base, 'select * from user'))



#!/usr/bin/env python
# coding=utf-8

"""
for example:
    from log_utils import LogUtils
    log = LogUtils('app.log', logging.DEBUG)
    log.debug('debug message')
    log.info('info message')
    log.warning('warning message')
    log.error('error message')
    >>>
    2022-01-01 00:00:00 - app.log - DEBUG - debug message
    2022-01-01 00:00:00 - app.log - INFO - info message
    2022-01-01 00:00:00 - app.log - WARNING - warning message
"""
import logging

class LogUtils:
    def __init__(self, log_file='app.log', name: str = 'root', log_level=logging.INFO):
        self.log_file = log_file
        self.log_level = log_level

        # 设置日志格式
        self.formatter = logging.Formatter(f'%(asctime)s - {name} - %(levelname)s - %(message)s')

        # 配置日志处理器
        self.file_handler = logging.FileHandler(self.log_file, encoding='utf-8')
        self.file_handler.setFormatter(self.formatter)

        # 配置控制台日志处理器
        self.console_handler = logging.StreamHandler()
        self.console_handler.setFormatter(self.formatter)

        # 获取根日志记录器并添加处理器
        self.logger = logging.getLogger()
        self.logger.setLevel(self.log_level)
        self.logger.addHandler(self.file_handler)
        self.logger.addHandler(self.console_handler)

    def get_logger(self):
        return self.logger

